﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter;
using GLAP.ViewModel;
using GLAP.Objects;

namespace GLAP
{
    /// <summary>
    /// Interaction logic for Support.xaml
    /// </summary>
    public partial class Support : Window
    {
        public static string RoundedCurrentTime;
        private SupportVM _vm;
        private int _windowBuffer;
        DateTime _currentTime;

        #region Constansts

        public const int TIME_INTERVAL = 15;               //This variable holds the time interval the application will display
        public const int COLUMNS_PER_HOUR = 60 / TIME_INTERVAL;
        public const int WINDOW_BUFFER_DEFAULT = 12 * (COLUMNS_PER_HOUR) + 2;        //4 is the number of hours to buffer to the left

        #endregion

        #region Accessors and Mutators

        public DateTime CurrentTime
        {
            get { return _currentTime; }
            set { _currentTime = value; }
        }

        public int WindowBuffer
        {
            get { return _windowBuffer; }
            set { _windowBuffer = value; }
        }

        #endregion


        public Support()
        {
            InitializeComponent();
            this.WindowState = WindowState.Maximized;

            WindowBuffer = WINDOW_BUFFER_DEFAULT;

            _vm = new SupportVM();
            DataContext = _vm;

            RoundedCurrentTime = roundedDate();
        }

        #region Highlight Column

        private void CellPresenter_MouseLeave(object sender, MouseEventArgs e)
        {
            CellValuePresenter cp = (CellValuePresenter)sender;

            if (cp.Field != null)
            {
                _vm.FieldName = string.Empty;
            }
        }

        private void CellPresenter_MouseEnter(object sender, MouseEventArgs e)
        {
            CellValuePresenter cp = (CellValuePresenter)sender;

            if (cp.Field != null)
            {
                _vm.FieldName = cp.Field.Name;
            }
        }

        private string roundedDate()
        {
            StringBuilder roundedDateTime = new StringBuilder();
            roundedDateTime.Append(_vm.CurrentTime.TimeOfDay.Hours.ToString("00"));
            roundedDateTime.Append(":");
            roundedDateTime.Append((_vm.CurrentTime.TimeOfDay.Minutes / MainWindow.TIME_INTERVAL * MainWindow.TIME_INTERVAL).ToString("00"));
            roundedDateTime.Append("/");
            roundedDateTime.Append(_vm.CurrentTime.Month.ToString("00"));
            roundedDateTime.Append("/");
            roundedDateTime.Append(_vm.CurrentTime.Day.ToString("00"));

            return roundedDateTime.ToString();
        }

        private void XamDataGrid_FieldLayoutInitialized(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            FieldInit(sender, e, true, xdgMainDisplay);
        }

        private void FieldInit(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e, bool childrenHaveChildren, XamDataGrid myGrid)
        {
            Forecast elem = _vm.TopNode.ElementAt(0).Child[0].Forecasts[0];
            _vm.RegularParentValueStyle = LayoutRoot.Resources["CVP_RegularParent"] as Style;
            _vm.EditedParentValueStyle = LayoutRoot.Resources["CVP_EditedParent"] as Style;
            _vm.RegularChildValueStyle = LayoutRoot.Resources["CVP_RegularChild"] as Style;
            _vm.EditedChildValueStyle = LayoutRoot.Resources["CVP_EditedChild"] as Style;
            _vm.RegularStyle = LayoutRoot.Resources["CP_RegularStyle"] as Style;

            _vm.FieldInit(sender, e, childrenHaveChildren, myGrid);
        }

        private void xdgMainDisplay_AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e)
        {
            AssigningFieldLayoutToItem(sender, e, xdgMainDisplay);
        }

        private void AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e, XamDataGrid myGrid)
        {
            FieldLayout resultLayout = new FieldLayout();
            Node myNode = e.Item as Node;

            if (e.ParentExpandableFieldRecord == null)
                if (myNode.Child.Count == 0)
                    resultLayout = myGrid.FieldLayouts["Child"];
                else
                    resultLayout = myGrid.FieldLayouts["Node"];
            else if (e.FieldLayout.Description.ToString().Equals("Forecasts"))
                resultLayout = myGrid.FieldLayouts["Forecasts"];
            else
                resultLayout = myGrid.FieldLayouts["Child"];

            e.FieldLayout = resultLayout;
        }
        #endregion
    }
}
