﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;
using GLAP.Objects;
using System.Windows;
using Infragistics.Windows.DataPresenter;
using System.Windows.Data;
using GLAP.Data;
using System.Reflection;

namespace GLAP.ViewModel
{
    public class SupportVM
    {
        public List<string> _keys { get; set; }
        string _fieldname = string.Empty;
        DateTime _currentTime;
        DateTime _bufferMin;
        DateTime _bufferMax;
        ObservableCollection<Node> _topNode = new ObservableCollection<Node>();
        Dictionary<Node, List<string>> _nodeGroupings = new Dictionary<Node, List<string>>();
        Type NullableDateTimeType = Type.GetType("System.Nullable`1[[System.DateTime, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089]]");
        Type NullableBoolType = Type.GetType("System.Nullable`1[[System.Boolean, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089]]");

        public DateTime CurrentTime
        {
            get { return _currentTime; }
            set
            {
                _currentTime = value;
                BufferMin = RoundDate(_currentTime.AddHours(-15));
                BufferMax = RoundDate(_currentTime.AddDays(2).AddHours(13));
            }
        }

        public DateTime BufferMin
        {
            get { return _bufferMin; }
            set { _bufferMin = value; }
        }

        public DateTime BufferMax
        {
            get { return _bufferMax; }
            set { _bufferMax = value; }
        }

        public String FieldName
        {
            get { return _fieldname; }
            set
            {
                _fieldname = value;
                NotifyPropertyChanged("FieldName");
            }
        }

        private void NotifyPropertyChanged(String propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private DateTime RoundDate(DateTime originalDate)
        {
            DateTime roundedDate = originalDate.AddMinutes(-originalDate.Minute); ;
            roundedDate = roundedDate.AddSeconds(-originalDate.Second);
            roundedDate = DateTime.Parse(roundedDate.ToString("MM/dd/yyyy HH:mm:ss"));

            return roundedDate;
        }

        public ObservableCollection<Node> TopNode
        {
            get { return _topNode; }
            set { _topNode = value; }
        }

        public Style RegularStyle { get; set; }
        public Style RegularParentValueStyle { get; set; }
        public Style EditedParentValueStyle { get; set; }
        public Style RegularChildValueStyle { get; set; }
        public Style EditedChildValueStyle { get; set; }
        public Style CalculatedStyle { get; set; }

                public SupportVM()
        {
            CurrentTime = System.DateTime.Now;
            GetForecasts(CurrentTime.AddHours(-15), CurrentTime.AddDays(2).AddHours(13));
        }

                private void GetForecasts(DateTime startDate, DateTime endDate)
                {
                    _keys = new List<string>();
                    List<Forecast> myForecasts = new List<Forecast>();

                    CreateGroups();

                    try
                    {
                        //Add a new column for each time returned
                        using (GLAPEntities myEntity = new GLAPEntities())
                        {
                            List<CapacityForecastInfo> capacityTemp = myEntity.sp_CapacityEditedData(startDate, endDate).ToList();
                            PopulateForecast<CapacityForecastInfo>(capacityTemp);
                        }
                    }
                    catch (System.Exception ex)
                    {
                        throw;
                    }
                }

                private void PopulateForecast<T>(List<T> list)
                {
                    string newLoadName;
                    int newLoadValue;
                    string overrideColumn;

                    if (list.Count == 0)
                        return;

                    try
                    {
                        foreach (PropertyInfo prop in list.First().GetType().GetProperties())
                        {
                            if (prop.PropertyType != NullableDateTimeType && prop.PropertyType != NullableBoolType && !prop.PropertyType.Name.Equals("String"))
                            {
                                Forecast ForecastedLoad = new Forecast() { Name = prop.Name, Loads = new List<Load>(), EntityType = list.First().GetType().Name };

                                foreach (T obj in list)
                                {
                                    try
                                    {
                                        //LoadName is stored as MM/DD/HH:MM
                                        newLoadName = DateTime.Parse(obj.GetType().GetProperty("ForecastDate").GetValue(obj, null).ToString()).ToString("HH:mm/MM/dd");

                                        if (newLoadName.Contains("00:00"))
                                            newLoadName = DateTime.Parse(obj.GetType().GetProperty("ForecastDate").GetValue(obj, null).ToString()).ToString("MM/dd");
                                    }
                                    catch
                                    {
                                        newLoadName = "Error";
                                    }

                                    try
                                    {
                                        //if the value is null return 0
                                        if (prop.GetValue(obj, null) != null)
                                            newLoadValue = int.Parse(prop.GetValue(obj, null).ToString());
                                        else
                                            newLoadValue = 0;
                                    }
                                    catch
                                    {
                                        newLoadValue = 0;
                                    }

                                    //The OverrideColumn property is null if it's not an ovverride or contains the name of the column that was overrode. 
                                    if (obj.GetType().GetProperty("OverrideColumn").GetValue(obj, null) != null)
                                        overrideColumn = obj.GetType().GetProperty("OverrideColumn").GetValue(obj, null).ToString();
                                    else
                                        overrideColumn = string.Empty;

                                    ForecastedLoad.Loads.Add(new Load { Name = newLoadName, Value = newLoadValue, OverrideColumn = overrideColumn });
                                }
                                AddToProperGroup(ForecastedLoad);
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                }

                private void CreateGroups()
                {
                    List<Node> allParents = new List<Node>();
                    try
                    {
                        //Add a new column for each time returned
                        using (GLAPEntities myEntity = new GLAPEntities())
                        {
                            List<DataHierarchy> dataHierarchy = myEntity.DataHierarchies.ToList();
                            dataHierarchy = dataHierarchy.OrderBy(o => o.SortOrder).ToList();

                            //First create a node for each parent and find which children it should have
                            foreach (DataHierarchy dh in dataHierarchy)
                            {
                                if (!dh.ShortName.Equals("TopNode") && dh.HasChildren.Value)
                                {
                                    List<string> newParent = new List<string>();
                                    Node newParentGroup = new Node() { Name = dh.LongName, IsExpanded = false };

                                    //Create a list of all the children the parent has
                                    List<DataHierarchy> children = dataHierarchy.Where(c => c.ParentID == dh.ID).ToList();
                                    foreach (DataHierarchy child in children)
                                    {
                                        newParent.Add(child.ShortName);
                                    }
                                    //Add the parent the the node dictionary
                                    _nodeGroupings.Add(newParentGroup, newParent);
                                    if (!dh.HasParent.Value)
                                    {
                                        //Add the parent to the top node if it doens't have a parent (highest level)
                                        _topNode.Add(newParentGroup);
                                    }
                                }
                            }

                            //Now complete the middle of the hierarchy for the records that have both children and parents (sub-levels)
                            foreach (DataHierarchy parentChild in dataHierarchy)
                            {
                                if (!parentChild.ShortName.Equals("TopNode") && parentChild.HasChildren.Value && parentChild.HasParent.Value)
                                {
                                    Node nodeToAddToParent = new Node();
                                    List<DataHierarchy> parents = dataHierarchy.Where(c => c.ID == parentChild.ParentID).ToList();
                                    foreach (DataHierarchy parent in parents)
                                    {
                                        //add that node to parent
                                        foreach (KeyValuePair<Node, List<string>> pair in _nodeGroupings)
                                        {
                                            if (pair.Key.Name.Equals(parent.LongName))
                                            {
                                                allParents.Add(pair.Key);
                                            }
                                            //get node of parentChild from _nodeGroupings
                                            if (pair.Key.Name.Equals(parentChild.LongName))
                                            {
                                                nodeToAddToParent = pair.Key;
                                            }
                                        }
                                        if (!nodeToAddToParent.Name.Equals(""))
                                        {
                                            //add node to parent
                                            foreach (Node p in allParents)
                                            {
                                                p.Child.Add(nodeToAddToParent);
                                            }
                                        }
                                        allParents.Clear();
                                    }
                                }
                            }
                        }
                    }
                    catch (System.Exception ex)
                    {
                        throw;
                    }
                }

        public void FieldInit(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e, bool childrenHaveChildren, XamDataGrid myGrid)
        {
            int colWidth = 45;
            Forecast elem = TopNode.ElementAt(0).Child[0].Forecasts[0];  
            Style editableField = new Style(typeof(CellValuePresenter));
            Style calculatedField = new Style(typeof(CellValuePresenter));
            editableField.Setters.Add(new Setter(CellValuePresenter.FontWeightProperty, FontWeights.Bold));
            editableField.Setters.Add(new Setter(CellValuePresenter.HorizontalContentAlignmentProperty, HorizontalAlignment.Right));
            calculatedField.Setters.Add(new Setter(CellValuePresenter.HorizontalContentAlignmentProperty, HorizontalAlignment.Right));
            int columnCounter = 0;

            for (int i = 0; i < elem.Loads.Count; i++)
            {
                UnboundField field = new UnboundField
                {
                    Name = elem.Loads[i].Name,
                    Label = elem.Loads[i].Name.Remove(5, elem.Loads[i].Name.Length - 5),
                    Width = new FieldLength(colWidth)
                };

                if (e.FieldLayout.Description.Equals("Forecasts"))
                {
                    if (columnCounter < 50)
                    {
                        field.Width = new FieldLength(colWidth + 17);
                        columnCounter++;
                    }
                    field.BindingPath = new PropertyPath("Loads[" + i + "].Value");

                    field.Settings.CellValuePresenterStyleSelector = new CVPStyleSelector_Field(RegularChildValueStyle, EditedChildValueStyle);
                }
                else
                {
                    Binding bind = new Binding();
                    bind.Converter = new FindSums2();
                    bind.ConverterParameter = i;
                    field.Binding = bind;

                    field.Settings.CellValuePresenterStyleSelector = new CVPStyleSelector_Field(RegularParentValueStyle, EditedParentValueStyle);

                }
                e.FieldLayout.Fields.Add(field);

            }
        }

        private void AddToProperGroup(Forecast newForecast)
        {
            //Get all Groups that have the 'newForecast' name
            //Bring back all keys (or Parents) where newForecast.Name exists
            try
            {
                var ProperGroups = (from i in _nodeGroupings
                                    where (from a in i.Value
                                           where a.Equals(newForecast.Name)
                                           select i.Key).Contains(i.Key)
                                    select i).ToList();

                //Add the forecast to all the groups
                foreach (var myGroup in ProperGroups)
                {
                    myGroup.Key.Forecasts.Add(newForecast);
                }
            }
            catch
            {
                throw;
            }
        }

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion
    }


}
